import React, {Component, PropTypes} from 'react';
import WebViewBridge from 'react-native-webview-bridge-updated';
import {InjectedMessageHandler} from './WebviewMessageHandler';
import {actions, messages} from './const';

const injectScript = `
  (function () {
    ${InjectedMessageHandler}
  }());
`;

export default class RichTextEditor extends Component {
  static propTypes = {
    initialHTML: PropTypes.string
  };

  constructor(props) {
    super(props);
    this._sendAction = this._sendAction.bind(this);
  }

  onBridgeMessage(message){
    try {
      const json = JSON.parse(message);

      switch (json.type) {
        case messages.HTML_RESPONSE:
          if (this.resolve) {
            this.resolve(json.data);
            this.resolve = undefined;
            this.reject = undefined;
            if (this.pendingHtml) {
              clearTimeout(this.pendingHtml);
              this.pendingHtml = undefined;
            }
          }
          break;
        case messages.ZSS_INITIALIZED:
          this.setHTML(this.props.initialHTML);
          break;
      }

    } catch(e) {
      //alert('NON JSON MESSAGE');
    }
  }

  render() {
    return (
      <WebViewBridge
        {...this.props}
        hideKeyboardAccessoryView={true}
        ref={(r) => {this.webviewBridge = r}}
        onBridgeMessage={(message) => this.onBridgeMessage(message)}
        injectedJavaScript={injectScript}
        source={require('./editor.html')}
      />
    );
  }

  _sendAction(action, data) {
    this.webviewBridge.sendToBridge(JSON.stringify({type: action, data}));
  }

  //-------------------------------------------------------------------------------
  //--------------- Public API

  setHTML(html) {
    this._sendAction(actions.setHtml, html);
  }

  blurEditor() {
    this._sendAction(actions.blurEditor);
  }

  setBold() {
    this._sendAction(actions.setBold);
  }

  setItalic() {
    this._sendAction(actions.setItalic);
  }

  setUnderline() {
    this._sendAction(actions.setUnderline);
  }

  heading1() {
    this._sendAction(actions.heading1);
  }

  heading2() {
    this._sendAction(actions.heading2);
  }

  heading3() {
    this._sendAction(actions.heading3);
  }

  heading4() {
    this._sendAction(actions.heading4);
  }

  heading5() {
    this._sendAction(actions.heading5);
  }

  heading6() {
    this._sendAction(actions.heading6);
  }

  setParagraph() {
    this._sendAction(actions.setParagraph);
  }

  removeFormat() {
    this._sendAction(actions.removeFormat);
  }

  alignLeft() {
    this._sendAction(actions.alignLeft);
  }

  alignCenter() {
    this._sendAction(actions.alignCenter);
  }

  alignRight() {
    this._sendAction(actions.alignRight);
  }

  alignFull() {
    this._sendAction(actions.alignFull);
  }

  insertBulletsList() {
    this._sendAction(actions.insertBulletsList);
  }

  insertOrderedList() {
    this._sendAction(actions.insertOrderedList);
  }

  setSubscript() {
    this._sendAction(actions.setSubscript);
  }

  setSuperscript() {
    this._sendAction(actions.setSuperscript);
  }

  setStrikethrough() {
    this._sendAction(actions.setStrikethrough);
  }

  setHR() {
    this._sendAction(actions.setHR);
  }

  setIndent() {
    this._sendAction(actions.setIndent);
  }

  setOutdent() {
    this._sendAction(actions.setOutdent);
  }

  setPlaceholder() {
    this._sendAction(actions.setPlaceholder);
  }

  async getHtml() {
    return new Promise((resolve, reject) => {
      this.resolve = resolve;
      this.reject = reject;
      this._sendAction(actions.getHtml);

      this.pendingHtml = setTimeout(() => {
        if (this.reject) {
          this.reject('timeout')
        }
      }, 5000);
    });
  }

}